@@ -12,11 +12,19 @@ import Network.HTTP2.H2.Types
 
 -- | Making simple configuration whose IO is not efficient.
 --   A write buffer is allocated internally.
+--   WAI timeout manger is initialized with 30_000_000 microseconds.
 allocSimpleConfig :: Socket -> BufferSize -> IO Config
-allocSimpleConfig s bufsiz = do
+allocSimpleConfig s bufsiz = allocSimpleConfig' s bufsiz (30 * 1000000)
+
+-- | Making simple configuration whose IO is not efficient.
+--   A write buffer is allocated internally.
+--   The third argument is microseconds to initialize WAI
+--   timeout manager.
+allocSimpleConfig' :: Socket -> BufferSize -> Int -> IO Config
+allocSimpleConfig' s bufsiz usec = do
     buf <- mallocBytes bufsiz
     ref <- newIORef Nothing
-    timmgr <- T.initialize $ 30 * 1000000
+    timmgr <- T.initialize usec
     mysa <- getSocketName s
     peersa <- getPeerName s
     let config =
