@@ -3,11 +3,15 @@
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE RecordWildCards #-}
 
-module HTTP2.ServerSpec where
+module HTTP2.ServerSpec (spec) where
 
 import Control.Concurrent
+-- cryptonite
+
+import Control.Concurrent.Async
+import qualified Control.Exception as E
 import Control.Monad
-import Crypto.Hash (Context, SHA1) -- cryptonite
+import Crypto.Hash (Context, SHA1)
 import qualified Crypto.Hash as CH
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
@@ -23,8 +27,6 @@ import System.IO
 import System.IO.Unsafe
 import System.Random
 import Test.Hspec
-import UnliftIO.Async
-import qualified UnliftIO.Exception as E
 
 import Network.HPACK
 import Network.HPACK.Internal
@@ -237,10 +239,10 @@ client3 sendRequest _aux = do
                 FileSpec "test/inputFile" 0 1012731
         req = C.setRequestTrailersMaker req0 maker
     sendRequest req $ \rsp -> do
-        let comsumeBody = do
+        let consumeBody = do
                 bs <- C.getResponseBodyChunk rsp
-                when (bs /= "") comsumeBody
-        comsumeBody
+                when (bs /= "") consumeBody
+        consumeBody
         mt <- C.getResponseTrailers rsp
         firstTrailerValue <$> mt `shouldBe` Just hx
   where
@@ -258,10 +260,10 @@ client3' sendRequest _aux = do
             withFile "test/inputFile" ReadMode sendFile
         req = C.setRequestTrailersMaker req0 maker
     sendRequest req $ \rsp -> do
-        let comsumeBody = do
+        let consumeBody = do
                 bs <- C.getResponseBodyChunk rsp
-                when (bs /= "") comsumeBody
-        comsumeBody
+                when (bs /= "") consumeBody
+        consumeBody
         mt <- C.getResponseTrailers rsp
         firstTrailerValue <$> mt `shouldBe` Just hx
   where
@@ -278,10 +280,10 @@ client3'' sendRequest _axu = do
             write $ byteString tag
         req = C.setRequestTrailersMaker req0 maker
     sendRequest req $ \rsp -> do
-        let comsumeBody = do
+        let consumeBody = do
                 bs <- C.getResponseBodyChunk rsp
-                when (bs /= "") comsumeBody
-        comsumeBody
+                when (bs /= "") consumeBody
+        consumeBody
         mt <- C.getResponseTrailers rsp
         firstTrailerValue <$> mt `shouldBe` Just hx
   where
