@@ -4,17 +4,26 @@
 
 module Main (main) where
 
+import Control.Concurrent
+import qualified Control.Exception as E
+import Control.Monad
 import Network.HTTP2.Server
 import Network.Run.TCP
 import System.Console.GetOpt
 import System.Environment
 import System.Exit
-import qualified UnliftIO.Exception as E
 
+import Monitor
 import Server
 
 options :: [OptDescr (Options -> Options)]
-options = []
+options =
+    [ Option
+        ['m']
+        ["monitor"]
+        (NoArg (\opts -> opts{optMonitor = True}))
+        "run thread monitor"
+    ]
 
 showUsageAndExit :: String -> IO a
 showUsageAndExit msg = do
@@ -28,23 +37,31 @@ serverOpts argv =
         (o, n, []) -> return (foldl (flip id) defaultOptions o, n)
         (_, _, errs) -> showUsageAndExit $ concat errs
 
-data Options = Options deriving (Show)
+data Options = Options
+    { optMonitor :: Bool
+    }
+    deriving (Show)
 
 defaultOptions :: Options
-defaultOptions = Options
+defaultOptions =
+    Options
+        { optMonitor = False
+        }
 
 usage :: String
 usage = "Usage: h2c-server [OPTION] <addr> <port>"
 
 main :: IO ()
 main = do
+    labelMe "h2c-server main"
     args <- getArgs
-    (Options, ips) <- serverOpts args
+    (opts, ips) <- serverOpts args
     (host, port) <- case ips of
         [h, p] -> return (h, p)
         _ -> showUsageAndExit usage
-    runTCPServer (Just host) port $ \s ->
+    when (optMonitor opts) $ void $ forkIO $ monitor $ threadDelay 1000000
+    runTCPServer (Just host) port $ \s -> do
         E.bracket
-            (allocSimpleConfig s 4096)
+            (allocSimpleConfig' s 4096 5000000)
             freeSimpleConfig
             (\conf -> run defaultServerConfig conf server)
